<?php

namespace Drupal\Tests\registration\Functional;

/**
 * Tests the registration administrative UI.
 *
 * @group registration
 */
class RegistrationAdminTest extends RegistrationBrowserTestBase {

  /**
   * Tests viewing a registration.
   */
  public function testView() {
    $user = $this->drupalCreateUser();
    $user->set('field_registration', 'conference');
    $user->save();

    $handler = $this->entityTypeManager->getHandler('registration', 'host_entity');
    $host_entity = $handler->createHostEntity($user);

    /** @var \Drupal\registration\RegistrationSettingsStorage $settings_storage */
    $settings_storage = $this->entityTypeManager->getStorage('registration_settings');
    $settings = $settings_storage->loadSettingsForHostEntity($host_entity);
    $settings->set('status', TRUE);
    $settings->save();

    /** @var \Drupal\registration\RegistrationStorage $registration_storage */
    $registration_storage = $this->entityTypeManager->getStorage('registration');
    $registration = $registration_storage->create([
      'workflow' => 'registration',
      'state' => 'pending',
      'type' => 'conference',
      'entity_type_id' => 'user',
      'entity_id' => $user->id(),
      'user_uid' => 1,
    ]);
    $registration->save();

    $this->drupalGet('/registration/' . $registration->id());
    $this->assertSession()->pageTextContains('Registration #' . $registration->id());
    $this->assertSession()->pageTextContains('Pending');
  }

  /**
   * Tests editing a registration.
   */
  public function testEdit() {
    $user = $this->drupalCreateUser();
    $user->set('field_registration', 'conference');
    $user->save();

    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $display_repository */
    $display_repository = \Drupal::service('entity_display.repository');
    $entity_form_display = $display_repository->getFormDisplay('registration', 'conference', 'default');
    $entity_form_display->setComponent('count', [
      'type' => 'number',
    ])->save();

    $handler = $this->entityTypeManager->getHandler('registration', 'host_entity');
    $host_entity = $handler->createHostEntity($user);

    /** @var \Drupal\registration\RegistrationSettingsStorage $settings_storage */
    $settings_storage = $this->entityTypeManager->getStorage('registration_settings');
    $settings = $settings_storage->loadSettingsForHostEntity($host_entity);
    $settings->set('status', TRUE);
    $settings->set('capacity', 0);
    $settings->set('maximum_spaces', 2);
    $settings->save();

    /** @var \Drupal\registration\RegistrationStorage $registration_storage */
    $registration_storage = $this->entityTypeManager->getStorage('registration');
    $registration = $registration_storage->create([
      'workflow' => 'registration',
      'state' => 'pending',
      'type' => 'conference',
      'entity_type_id' => 'user',
      'entity_id' => $user->id(),
      'user_uid' => 1,
      'langcode' => 'en',
    ]);
    $registration->save();

    $admin_user = $this->drupalCreateUser([
      'access user profiles',
      'administer registration',
      'create conference registration other users',
    ]);
    $this->drupalLogin($admin_user);
    $this->drupalGet('/registration/' . $registration->id() . '/edit');
    $this->assertSession()->pageTextContains('Edit Registration #' . $registration->id());
    $this->assertSession()->fieldEnabled('count[0][value]');
    $this->assertSession()->pageTextContainsOnce('The number of spaces you wish to reserve. You may register up to 2 spaces.');
    $this->assertSession()->buttonExists('Save Registration');
    $this->getSession()->getPage()->pressButton('Save Registration');
    $this->assertSession()->pageTextContains('Registration has been saved.');

    // When only one space can be registered, the Spaces field is hidden.
    $settings->set('maximum_spaces', 1);
    $settings->save();
    $this->drupalGet('/registration/' . $registration->id() . '/edit');
    $this->assertSession()->pageTextContains('Edit Registration #' . $registration->id());
    $this->assertSession()->buttonExists('Save Registration');
    $this->assertSession()->fieldNotExists('count[0][value]');
    $this->assertSession()->pageTextNotContains('The number of spaces you wish to reserve.');

    // No valid registration options.
    $admin_user = $this->drupalCreateUser([
      'access user profiles',
      'administer registration',
      'create conference registration other anonymous',
    ]);
    $this->drupalLogin($admin_user);
    $this->drupalGet('/registration/' . $registration->id() . '/edit');
    $this->assertSession()->pageTextContains('Edit Registration #' . $registration->id());
    $this->assertSession()->buttonNotExists('Save Registration');
    $this->assertSession()->pageTextContainsOnce('No valid registration options exist. Registration permissions may need to be adjusted.');

    // Editing a registration for anonymous.
    $registration->set('user_uid', NULL);
    $registration->set('anon_mail', 'someone@example.org');
    $registration->save();
    $this->drupalGet('/registration/' . $registration->id() . '/edit');
    $this->assertSession()->pageTextContains('Edit Registration #' . $registration->id());
    $this->assertSession()->buttonExists('Save Registration');
    $this->assertSession()->pageTextNotContains('No valid registration options exist. Registration permissions may need to be adjusted.');

    // No valid registration options.
    $admin_user = $this->drupalCreateUser([
      'access user profiles',
      'administer registration',
      'create conference registration other users',
    ]);
    $this->drupalLogin($admin_user);
    $this->drupalGet('/registration/' . $registration->id() . '/edit');
    $this->assertSession()->pageTextContains('Edit Registration #' . $registration->id());
    $this->assertSession()->buttonNotExists('Save Registration');
    $this->assertSession()->pageTextContainsOnce('No valid registration options exist. Registration permissions may need to be adjusted.');
  }

  /**
   * Tests deleting a registration.
   */
  public function testDelete() {
    $user = $this->drupalCreateUser();
    $user->set('field_registration', 'conference');
    $user->save();

    $handler = $this->entityTypeManager->getHandler('registration', 'host_entity');
    $host_entity = $handler->createHostEntity($user);

    /** @var \Drupal\registration\RegistrationSettingsStorage $settings_storage */
    $settings_storage = $this->entityTypeManager->getStorage('registration_settings');
    $settings = $settings_storage->loadSettingsForHostEntity($host_entity);
    $settings->set('status', TRUE);
    $settings->save();

    /** @var \Drupal\registration\RegistrationStorage $registration_storage */
    $registration_storage = $this->entityTypeManager->getStorage('registration');
    $registration = $registration_storage->create([
      'workflow' => 'registration',
      'state' => 'pending',
      'type' => 'conference',
      'entity_type_id' => 'user',
      'entity_id' => $user->id(),
      'user_uid' => 1,
      'langcode' => 'en',
    ]);
    $registration->save();

    $this->drupalGet('/registration/' . $registration->id() . '/delete');
    $this->assertSession()->pageTextContains('Are you sure you want to delete the registration Registration #' . $registration->id());
    $this->assertSession()->pageTextContains('This action cannot be undone.');
    $this->getSession()->getPage()->pressButton('Delete');
    $this->assertSession()->pageTextContains('has been deleted.');
  }

}
